"""Module with some color helpers"""

COLOR_BLACK = (0, 0, 0)
COLOR_WHITE = (255, 255, 255)


class Palette:
    """Class representing standard colors"""
    BLACK = COLOR_BLACK
    WHITE = COLOR_WHITE


__all__ = ["Palette", "COLOR_BLACK", "COLOR_WHITE"]
